#include "Vector.hpp"

#include <cmath>
#include <algorithm>

#include "Utils.hpp"

Vector::Vector()
    : x(0)
    , y(0)
{}

Vector::Vector(double _x, double _y) 
    : x(_x)
    , y(_y) 
{}

Vector::Vector(const Vector& o) 
    : x(o.x)
    , y(o.y) 
{}

Vector Vector::rand() {
    return Vector(drand(), drand());
}

double Vector::dot(const Vector& o) const {
    return o.x * x + o.y * y;
}

Vector Vector::projectOnto(const Vector& o) const {
    return o * dot(o);
}

Vector Vector::leftOrtho() const {
    return Vector(y, -x);
}

Vector Vector::operator* (double fac) const {
    return Vector(x * fac, y * fac);
}

Vector Vector::operator+ (const Vector& o) const {
    return Vector(x + o.x, y + o.y);
}

void Vector::operator+= (const Vector& o) {
    x += o.x; y += o.y;
}

void Vector::operator-= (const Vector& o) {
    x -= o.x; y -= o.y;
}

void Vector::operator*= (double o) {
    x *= o; y *= o;
}

void Vector::operator*= (const Vector& o) {
    x *= o.x; y *= o.y;
}

Vector Vector::operator- () const {
    return Vector(-x, -y);
}

Vector Vector::operator- (Vector o) const {
    return Vector(x - o.x, y - o.y);
}

void Vector::operator= (const Vector& o) {
    x = o.x; y = o.y;
}

double Vector::len2() const {
    return x * x + y * y;
}

double Vector::len() const {
    return sqrt(len2());
}

Vector Vector::operator/ (double quo) const {
    return Vector(x / quo, y / quo);
}

Vector Vector::normalize() const {
    return *this / len();
}

double Vector::angle() const {
    return std::atan2(y, x);
}

Area::Area(const Vector& _p1, const Vector& _p2) 
    : p1(_p1)
    , p2(_p2) {
}

double Area::area() const {
    return (p2.x - p1.x) * (p2.y - p1.y);
}

Area Area::intersect(const Area& o) const {
    return Area(Vector(fmax(p1.x, o.p1.x), fmax(p1.y, o.p1.y)),
                Vector(fmin(p2.x, o.p2.x), fmin(p2.y, o.p2.y)));
}
