#ifndef _DOMAIN_HPP
#define _DOMAIN_HPP

#include "Config.hpp"
#include "QuickSet.hpp"
#include "Particle.hpp"
#include "Simulation.hpp"

void intersect(const Vector& p1, const Vector& d1, const Vector& p2, const Vector& d2, double& k1, double& k2);

struct Domain {
    Domain();

    /** Simulate all particles of this domain. */
    void tick(double dt);

    /** Move particles that are not inside this domain anymore to different Domains */
    void transfer();

    /** Save all particles that were in this domain at interation start into file. */
    void write(int offsetOrTotalCount);

    /** Insert particle with position and velocity. */
    void insert(const Vector& pos, const Vector& vel = Vector());

    /** Send my data to the remote now. */
    void sendTo(int remote);

    /** Receive new data from remote. */
    void receiveFrom(int remote);

    /** Broadcast all data across the network, from rank `root` to all others. Either sends or receives. */
    void broadcast(int root);

    /** Removes all particles. */
    void clear();

    /** Removes a particle. */
    void kill(int i);

    /** Counts number of particles. */
    unsigned int count() const;

    // Particle data
    QuickSet<Vector> p_pos; // positions
    QuickSet<Vector> p_vel; // velocities
    QuickSet<Vector> p_frc; // forces
};

#endif