#include "Grid.hpp"

#include "Config.hpp"
#include <cstdlib>

Grid::Grid() {
    col_start = (double*)calloc(config.gridColumns, sizeof(double));
    row_start = (double*)calloc(config.gridRows, sizeof(double));

    // start with uniform distribution
    for(int i = 0; i < config.gridColumns; ++i) {
        col_start[i] = 1.0 * i / config.gridColumns;
    }
    for(int i = 0; i < config.gridRows; ++i) {
        row_start[i] = 1.0 * i / config.gridRows;
    }
}

int Grid::getDomain(const Vector& pos) const {
    int row = config.gridRows - 1;
    int col = config.gridColumns - 1;
    while(col > 0 && col_start[col] > pos.x) {
        col -= 1;
    }
    while(row > 0 && row_start[row] > pos.y) {
        row -= 1;
    }
    return col + row * config.gridColumns;
}

void Grid::getPosition(int& x, int& y) {
    x = config.rank % config.gridColumns;
    y = config.rank / config.gridColumns;
}

int Grid::getRank(int x, int y) {
    if(x < 0 || x >= config.gridColumns || y < 0 || y >= config.gridRows) return -1;
    return x + y * config.gridColumns;
}
