#include "Mesh.hpp"
#include "MeshFragment.hpp"

Mesh::Mesh(int count, double radius) 
    : fragments(count)
    , radius2(radius * radius)
{}

void Mesh::addPoint(const Vector& point) {
    points.push_back(point);
}

void Mesh::finish(bool close) {
    if(close) {
        points.push_back(points[0]);
    }

    auto prev = points.begin();
    int id = 0;
    for(auto i = points.begin() + 1; i != points.end(); ++i) {
        MeshFragment fragment;
        fragment.id = id++;
        fragment.pos = *prev;
        fragment.dir = *i - *prev;
        fragment.nor = fragment.dir.leftOrtho().normalize();
        fragments.insert(fragment);
        prev = i;
    }
}

bool Mesh::isOutside(const Vector& p) {
    for(int i = 0; i < fragments.count; ++i) {
        MeshFragment& f = fragments[i];
        bool isLeft = (f.dir.x * (p.y - f.pos.y) - f.dir.y * (p.x - f.pos.x)) > 0;
        // printf("Is Left of fragment %i? %d\n", i, isLeft);
        if(!isLeft) {
            // printf("Fragment info: %.2f %.2f / %.2f %.2f\n", f.pos.x, f.pos.y, f.dir.x, f.dir.y);
            return true;
        }
    }
    return false;
}