#ifndef _QUICKSET_HPP
#define _QUICKSET_HPP

#include <stdio.h> 
#include <cstdlib> 
#include <cassert>
#include <cstring>
#include "Config.hpp"

template <typename T>
struct QuickSet {
    QuickSet(unsigned int s) 
        : size(s)
        , count(0) {
        buf = (T*)calloc(size, sizeof(T));
        if(buf == 0) {
            printf("Out of memory.\n");
            exit(1);
        }
    }

    void remove(unsigned int index) {
        if(index >= count || index < 0) {
            return;
        }

        // move last element to index position
        memcpy(&buf[index], &buf[count-1], sizeof(T));
        // shorten array by one
        count--;
    }

    T* allocate() {
        assert(count < size);
        count++;
        return &buf[count-1];
    }

    bool insert(const T& item) {
        if(count >= size) {
            // printf("Count %#010x > Size %#010x\n", count, size);
            assert(!config.debug);
            return false;
        }
        T* t = allocate();
        memcpy(t, &item, sizeof(T));
        return true;
    }

    T& operator[](int index) {
        if(index >= size || index < 0) {
            printf("Index %d out of bounds.\n", index);
            assert(!config.debug);
            return default_return;
        }
        return buf[index];   
    }

    void* ptr() {
        return (void*)buf;
    }

    void clear() {
        count = 0;
    }

    // Members
    unsigned int size;
    unsigned int count = 0;
    T* buf;

    T default_return;
};

#endif